@@ -3,12 +3,13 @@ extern crate num;
 
 use dim4::Dim4;
 use array::Array;
-use defines::{AfError, DType, Scalar};
+use defines::{AfError};
 use error::HANDLE_ERROR;
 use self::libc::{c_int};
-use data::{constant, constant_t, tile};
+use data::{constant, ConstGenerator, tile};
 use self::num::Complex;
-use util::{AfArray, Complex32, Complex64, MutAfArray};
+use num::Zero;
+use util::{AfArray, HasAfEnum, ImplicitPromote, MutAfArray};
 use std::ops::Neg;
 use std::ops::{Add, Sub, Div, Mul, BitAnd, BitOr, BitXor, Not, Rem, Shl, Shr};
 
@@ -94,77 +95,106 @@ extern {
 }
 
 /// Enables use of `!` on objects of type [Array](./struct.Array.html)
-impl<'f> Not for &'f Array {
-    type Output = Array;
+impl<'f, T> Not for &'f Array<T>
+    where T: HasAfEnum
+{
+    type Output = Array<T>;
 
-    fn not(self) -> Array {
+    fn not(self) -> Self::Output {
+        let mut temp: i64 = 0;
         unsafe {
-            let mut temp: i64 = 0;
-            let err_val = af_not(&mut temp as MutAfArray, self.get() as AfArray);
+            let err_val = af_not(&mut temp as MutAfArray,
+                                 self.get() as AfArray);
             HANDLE_ERROR(AfError::from(err_val));
-            Array::from(temp)
         }
+        temp.into()
     }
 }
 
 macro_rules! unary_func {
-    ($doc_str: expr, $fn_name: ident, $ffi_fn: ident) => (
+    [$doc_str: expr, $fn_name: ident, $ffi_fn: ident, $out_type: ident] => (
         #[doc=$doc_str]
         ///
         /// This is an element wise unary operation.
         #[allow(unused_mut)]
-        pub fn $fn_name(input: &Array) -> Array {
+        pub fn $fn_name<T: HasAfEnum>(input: &Array<T>) -> Array< T::$out_type >
+        where T::$out_type: HasAfEnum {
+            let mut temp: i64 = 0;
             unsafe {
-                let mut temp: i64 = 0;
                 let err_val = $ffi_fn(&mut temp as MutAfArray, input.get() as AfArray);
                 HANDLE_ERROR(AfError::from(err_val));
-                Array::from(temp)
             }
+            temp.into()
         }
     )
 }
 
-unary_func!("Computes absolute value", abs, af_abs);
-unary_func!("Computes phase value", arg, af_arg);
-unary_func!("Computes the sign of input Array values", sign, af_sign);
-unary_func!("Round the values in an Array", round, af_round);
-unary_func!("Truncate the values in an Array", trunc, af_trunc);
-unary_func!("Floor the values in an Array", floor, af_floor);
-unary_func!("Ceil the values in an Array", ceil, af_ceil);
-unary_func!("Compute sin", sin, af_sin);
-unary_func!("Compute cos", cos, af_cos);
-unary_func!("Compute tan", tan, af_tan);
-unary_func!("Compute asin", asin, af_asin);
-unary_func!("Compute acos", acos, af_acos);
-unary_func!("Compute atan", atan, af_atan);
-unary_func!("Create a complex Array from real Array", cplx, af_cplx);
-unary_func!("Extract real values from a complex Array", real, af_real);
-unary_func!("Extract imaginary values from a complex Array", imag, af_imag);
-unary_func!("Compute the complex conjugate", conjg, af_conjg);
-unary_func!("Compute sinh", sinh, af_sinh);
-unary_func!("Compute cosh", cosh, af_cosh);
-unary_func!("Compute tanh", tanh, af_tanh);
-unary_func!("Compute asinh", asinh, af_asinh);
-unary_func!("Compute acosh", acosh, af_acosh);
-unary_func!("Compute atanh", atanh, af_atanh);
-unary_func!("Compute two raised to the power of value", pow2, af_pow2);
-unary_func!("Compute e raised to the power of value", exp, af_exp);
-unary_func!("Compute sigmoid function", sigmoid, af_sigmoid);
-unary_func!("Compute e raised to the power of value -1", expm1, af_expm1);
-unary_func!("Compute error function value", erf, af_erf);
-unary_func!("Compute the complementary error function value", erfc, af_erfc);
-unary_func!("Compute the natural logarithm", log, af_log);
-unary_func!("Compute the logarithm of input Array + 1", log1p, af_log1p);
-unary_func!("Compute logarithm base 10", log10, af_log10);
-unary_func!("Compute logarithm base 2", log2, af_log2);
-unary_func!("Compute the square root", sqrt, af_sqrt);
-unary_func!("Compute the cube root", cbrt, af_cbrt);
-unary_func!("Compute the factorial", factorial, af_factorial);
-unary_func!("Compute gamma function", tgamma, af_tgamma);
-unary_func!("Compute the logarithm of absolute values of gamma function", lgamma, af_lgamma);
-unary_func!("Check if values are zero", iszero, af_iszero);
-unary_func!("Check if values are infinity", isinf, af_isinf);
-unary_func!("Check if values are NaN", isnan, af_isnan);
+unary_func!("Computes absolute value", abs, af_abs, AbsOutType);
+unary_func!("Computes phase value", arg, af_arg, ArgOutType);
+
+unary_func!("Truncate the values in an Array", trunc, af_trunc, AbsOutType);
+unary_func!("Computes the sign of input Array values", sign, af_sign, AbsOutType);
+unary_func!("Round the values in an Array", round, af_round, AbsOutType);
+unary_func!("Floor the values in an Array", floor, af_floor, AbsOutType);
+unary_func!("Ceil the values in an Array", ceil, af_ceil, AbsOutType);
+
+unary_func!("Compute sigmoid function", sigmoid, af_sigmoid, AbsOutType);
+unary_func!("Compute e raised to the power of value -1", expm1, af_expm1, AbsOutType);
+unary_func!("Compute error function value", erf, af_erf, AbsOutType);
+unary_func!("Compute the complementary error function value", erfc, af_erfc, AbsOutType);
+
+unary_func!("Compute logarithm base 10", log10, af_log10, AbsOutType);
+unary_func!("Compute the logarithm of input Array + 1", log1p, af_log1p, AbsOutType);
+unary_func!("Compute logarithm base 2", log2, af_log2, AbsOutType);
+
+unary_func!("Compute the cube root", cbrt, af_cbrt, AbsOutType);
+unary_func!("Compute gamma function", tgamma, af_tgamma, AbsOutType);
+unary_func!("Compute the logarithm of absolute values of gamma function",
+            lgamma, af_lgamma, AbsOutType);
+
+unary_func!("Compute acosh", acosh, af_acosh, UnaryOutType);
+unary_func!("Compute acos", acos, af_acos, UnaryOutType);
+unary_func!("Compute asin", asin, af_asin, UnaryOutType);
+unary_func!("Compute asinh", asinh, af_asinh, UnaryOutType);
+unary_func!("Compute atan", atan, af_atan, UnaryOutType);
+unary_func!("Compute atanh", atanh, af_atanh, UnaryOutType);
+unary_func!("Compute cos", cos, af_cos, UnaryOutType);
+unary_func!("Compute cosh", cosh, af_cosh, UnaryOutType);
+unary_func!("Compute e raised to the power of value", exp, af_exp, UnaryOutType);
+unary_func!("Compute the natural logarithm", log, af_log, UnaryOutType);
+unary_func!("Compute sin", sin, af_sin, UnaryOutType);
+unary_func!("Compute sinh", sinh, af_sinh, UnaryOutType);
+unary_func!("Compute the square root", sqrt, af_sqrt, UnaryOutType);
+unary_func!("Compute tan", tan, af_tan, UnaryOutType);
+unary_func!("Compute tanh", tanh, af_tanh, UnaryOutType);
+
+unary_func!("Extract real values from a complex Array", real, af_real, AbsOutType);
+unary_func!("Extract imaginary values from a complex Array", imag, af_imag, AbsOutType);
+unary_func!("Create a complex Array from real Array", cplx, af_cplx, ComplexOutType);
+unary_func!("Compute the complex conjugate", conjg, af_conjg, ComplexOutType);
+unary_func!("Compute two raised to the power of value", pow2, af_pow2, UnaryOutType);
+unary_func!("Compute the factorial", factorial, af_factorial, UnaryOutType);
+
+macro_rules! unary_boolean_func {
+    [$doc_str: expr, $fn_name: ident, $ffi_fn: ident] => (
+        #[doc=$doc_str]
+        ///
+        /// This is an element wise unary operation.
+        #[allow(unused_mut)]
+        pub fn $fn_name<T: HasAfEnum>(input: &Array<T>) -> Array<bool> {
+            let mut temp: i64 = 0;
+            unsafe {
+                let err_val = $ffi_fn(&mut temp as MutAfArray, input.get() as AfArray);
+                HANDLE_ERROR(AfError::from(err_val));
+            }
+            temp.into()
+        }
+    )
+}
+
+unary_boolean_func!("Check if values are zero", iszero, af_iszero);
+unary_boolean_func!("Check if values are infinity", isinf, af_isinf);
+unary_boolean_func!("Check if values are NaN", isnan, af_isnan);
 
 macro_rules! binary_func {
     ($doc_str: expr, $fn_name: ident, $ffi_fn: ident) => (
@@ -172,15 +202,21 @@ macro_rules! binary_func {
         ///
         /// This is an element wise binary operation.
         #[allow(unused_mut)]
-        pub fn $fn_name(lhs: &Array, rhs: &Array, batch: bool) -> Array {
+        pub fn $fn_name<A, B>(lhs: &Array<A>,
+                              rhs: &Array<B>,
+                              batch: bool) -> Array< A::Output >
+            where A: HasAfEnum + ImplicitPromote<B>,
+                  B: HasAfEnum + ImplicitPromote<A>,
+                  <A as ImplicitPromote<B>>::Output: HasAfEnum
+        {
+            let mut temp: i64 = 0;
             unsafe {
-                let mut temp: i64 = 0;
                 let err_val = $ffi_fn(&mut temp as MutAfArray,
                                       lhs.get() as AfArray, rhs.get() as AfArray,
                                       batch as c_int);
                 HANDLE_ERROR(AfError::from(err_val));
-                Array::from(temp)
             }
+            Into::< Array< A::Output> >::into(temp)
         }
     )
 }
@@ -217,14 +253,22 @@ binary_func!("Compute length of hypotenuse of two Arrays", hypot, af_hypot);
 /// - u16
 ///
 pub trait Convertable {
-    /// Get an Array from given type
-    fn convert(&self) -> Array;
+    /// This type alias always points to `Self` which is the
+    /// type of [Array](./struct.Array.html) returned by the
+    /// trait method [convert](./trait.Convertable.html#tymethod.convert).
+    type OutType;
+
+    /// Get an Array of implementors type
+    fn convert(&self) -> Array< Self::OutType >
+        where <Self as Convertable>::OutType: HasAfEnum;
 }
 
 macro_rules! convertable_type_def {
     ($rust_type: ty) => (
         impl Convertable for $rust_type {
-            fn convert(&self) -> Array {
+            type OutType = $rust_type;
+
+            fn convert(&self) -> Array< Self::OutType > {
                 constant(*self, Dim4::new(&[1,1,1,1]))
             }
         }
@@ -239,25 +283,36 @@ convertable_type_def!(f64);
 convertable_type_def!(f32);
 convertable_type_def!(i32);
 convertable_type_def!(u32);
+convertable_type_def!(i16);
+convertable_type_def!(u16);
 convertable_type_def!(u8);
+convertable_type_def!(bool);
 
-impl Convertable for Array {
-    fn convert(&self) -> Array {
+impl<T: HasAfEnum> Convertable for Array<T> {
+    type OutType = T;
+
+    fn convert(&self) -> Array< Self::OutType > {
         self.clone()
     }
 }
 
 macro_rules! overloaded_binary_func {
     ($doc_str: expr, $fn_name: ident, $help_name: ident, $ffi_name: ident) => (
-        fn $help_name(lhs: &Array, rhs: &Array, batch: bool) -> Array {
+        fn $help_name<A, B>(lhs: &Array<A>,
+                            rhs: &Array<B>,
+                            batch: bool) -> Array< A::Output >
+            where A: HasAfEnum + ImplicitPromote<B>,
+                  B: HasAfEnum + ImplicitPromote<A>,
+                  <A as ImplicitPromote<B>>::Output: HasAfEnum
+        {
+            let mut temp: i64 = 0;
             unsafe {
-                let mut temp: i64 = 0;
                 let err_val = $ffi_name(&mut temp as MutAfArray,
                                      lhs.get() as AfArray, rhs.get() as AfArray,
                                      batch as c_int);
                 HANDLE_ERROR(AfError::from(err_val));
-                Array::from(temp)
             }
+            temp.into()
         }
 
         #[doc=$doc_str]
@@ -291,9 +346,16 @@ macro_rules! overloaded_binary_func {
         ///# Note
         ///
         /// The trait `Convertable` essentially translates to a scalar native type on rust or Array.
-        pub fn $fn_name<T, U> (arg1: &T, arg2: &U, batch: bool) -> Array where T: Convertable, U: Convertable {
-            let lhs = arg1.convert();
-            let rhs = arg2.convert();
+        pub fn $fn_name<T, U> (arg1: &T, arg2: &U, batch: bool)
+            -> Array< <<T as Convertable>::OutType as ImplicitPromote<<U as Convertable>::OutType>>::Output >
+                where T: Convertable,
+                      U: Convertable,
+                      <T as Convertable>::OutType: HasAfEnum + ImplicitPromote<<U as Convertable>::OutType>,
+                      <U as Convertable>::OutType: HasAfEnum + ImplicitPromote<<T as Convertable>::OutType>,
+                      <<T as Convertable>::OutType as ImplicitPromote<<U as Convertable>::OutType>>::Output: HasAfEnum
+        {
+            let lhs = arg1.convert(); // Convert to Array<T>
+            let rhs = arg2.convert(); // Convert to Array<T>
             match (lhs.is_scalar(), rhs.is_scalar()) {
                 ( true, false) => {
                     let l = tile(&lhs, rhs.dims());
@@ -309,8 +371,6 @@ macro_rules! overloaded_binary_func {
     )
 }
 
-// thanks to Umar Arshad for the idea on how to
-// implement overloaded function
 overloaded_binary_func!("Addition of two Arrays", add, add_helper, af_add);
 overloaded_binary_func!("Subtraction of two Arrays", sub, sub_helper, af_sub);
 overloaded_binary_func!("Multiplication of two Arrays", mul, mul_helper, af_mul);
@@ -329,6 +389,22 @@ overloaded_binary_func!("Create complex array from two Arrays", cplx2, cplx2_hel
 overloaded_binary_func!("Compute root", root, root_helper, af_root);
 overloaded_binary_func!("Computer power", pow, pow_helper, af_pow);
 
+fn clamp_helper<X, Y>(inp: &Array<X>, lo: &Array<Y>, hi: &Array<Y>, batch: bool)
+    -> Array< <X as ImplicitPromote<Y>>::Output >
+        where X: HasAfEnum + ImplicitPromote<Y>,
+              Y: HasAfEnum + ImplicitPromote<X>,
+              <X as ImplicitPromote<Y>>::Output: HasAfEnum
+{
+    let mut temp: i64 = 0;
+    unsafe {
+        let err_val = af_clamp(&mut temp as MutAfArray, inp.get() as AfArray,
+                               lo.get() as AfArray, hi.get() as AfArray,
+                               batch as c_int);
+        HANDLE_ERROR(AfError::from(err_val));
+    }
+    temp.into()
+}
+
 /// Clamp the values of Array
 ///
 /// # Parameters
@@ -347,50 +423,56 @@ overloaded_binary_func!("Computer power", pow, pow_helper, af_pow);
 /// # Note
 ///
 /// The trait `Convertable` essentially translates to a scalar native type on rust or Array.
-pub fn clamp<T, U> (input: &Array, arg1: &T, arg2: &U, batch: bool) -> Array
-    where T: Convertable, U: Convertable
+pub fn clamp<T, C>(input: &Array<T>, arg1: &C, arg2: &C, batch: bool)
+    -> Array< <T as ImplicitPromote<<C as Convertable>::OutType>>::Output >
+        where T: HasAfEnum + ImplicitPromote<<C as Convertable>::OutType>,
+              C: Convertable,
+              <C as Convertable>::OutType: HasAfEnum + ImplicitPromote<T>,
+              <T as ImplicitPromote<<C as Convertable>::OutType>>::Output: HasAfEnum
 {
-    let clamp_helper = |lo: &Array, hi: &Array| {
-        unsafe {
-            let mut temp: i64 = 0;
-            let err_val = af_clamp(&mut temp as MutAfArray, input.get() as AfArray,
-                                   lo.get() as AfArray, hi.get() as AfArray,
-                                   batch as c_int);
-            HANDLE_ERROR(AfError::from(err_val));
-            Array::from(temp)
-        }
-    };
-
-    let lo = arg1.convert();
-    let hi = arg2.convert();
+    let lo = arg1.convert(); // Convert to Array<T>
+    let hi = arg2.convert(); // Convert to Array<T>
     match (lo.is_scalar(), hi.is_scalar()) {
         ( true, false) => {
             let l = tile(&lo, hi.dims());
-            clamp_helper(&l, &hi)
+            clamp_helper(&input, &l, &hi, batch)
         },
         (false,  true) => {
             let r = tile(&hi, lo.dims());
-            clamp_helper(&lo, &r)
+            clamp_helper(&input, &lo, &r, batch)
         },
-        _ => clamp_helper(&lo, &hi),
+        ( true,  true) => {
+            let l = tile(&lo, input.dims());
+            let r = tile(&hi, input.dims());
+            clamp_helper(&input, &l, &r, batch)
+        },
+        _ => clamp_helper(&input, &lo, &hi, batch),
     }
 }
 
 macro_rules! arith_scalar_func {
     ($rust_type: ty, $op_name:ident, $fn_name: ident) => (
-        impl<'f> $op_name<$rust_type> for &'f Array {
-            type Output = Array;
+        impl<'f, T> $op_name<$rust_type> for &'f Array<T>
+            where T: HasAfEnum + ImplicitPromote<$rust_type>,
+                  $rust_type: HasAfEnum + ImplicitPromote<T>,
+                  <T as ImplicitPromote<$rust_type>>::Output: HasAfEnum
+        {
+            type Output = Array< <T as ImplicitPromote<$rust_type>>::Output >;
 
-            fn $fn_name(self, rhs: $rust_type) -> Array {
+            fn $fn_name(self, rhs: $rust_type) -> Self::Output {
                 let temp = rhs.clone();
                 $fn_name(self, &temp, false)
             }
         }
 
-        impl $op_name<$rust_type> for Array {
-            type Output = Array;
+        impl<T: HasAfEnum> $op_name<$rust_type> for Array<T>
+            where T: HasAfEnum + ImplicitPromote<$rust_type>,
+                  $rust_type: HasAfEnum + ImplicitPromote<T>,
+                  <T as ImplicitPromote<$rust_type>>::Output: HasAfEnum
+        {
+            type Output = Array< <T as ImplicitPromote<$rust_type>>::Output >;
 
-            fn $fn_name(self, rhs: $rust_type) -> Array {
+            fn $fn_name(self, rhs: $rust_type) -> Self::Output {
                 let temp = rhs.clone();
                 $fn_name(&self, &temp, false)
             }
@@ -419,34 +501,50 @@ arith_scalar_spec!(u8);
 
 macro_rules! arith_func {
     ($op_name:ident, $fn_name:ident, $delegate:ident) => (
-        impl $op_name<Array> for Array {
-            type Output = Array;
+        impl<A, B> $op_name<Array<B>> for Array<A>
+            where A: HasAfEnum + ImplicitPromote<B>,
+                  B: HasAfEnum + ImplicitPromote<A>,
+                  <A as ImplicitPromote<B>>::Output: HasAfEnum
+        {
+            type Output = Array< <A as ImplicitPromote<B>>::Output >;
 
-            fn $fn_name(self, rhs: Array) -> Array {
+            fn $fn_name(self, rhs: Array<B>) -> Self::Output {
                 $delegate(&self, &rhs, false)
             }
         }
 
-        impl<'a> $op_name<&'a Array> for Array {
-            type Output = Array;
+        impl<'a, A, B> $op_name<&'a Array<B>> for Array<A>
+            where A: HasAfEnum + ImplicitPromote<B>,
+                  B: HasAfEnum + ImplicitPromote<A>,
+                  <A as ImplicitPromote<B>>::Output: HasAfEnum
+        {
+            type Output = Array< <A as ImplicitPromote<B>>::Output >;
 
-            fn $fn_name(self, rhs: &'a Array) -> Array {
+            fn $fn_name(self, rhs: &'a Array<B>) -> Self::Output {
                 $delegate(&self, rhs, false)
             }
         }
 
-        impl<'a> $op_name<Array> for &'a Array {
-            type Output = Array;
+        impl<'a, A, B> $op_name<Array<B>> for &'a Array<A>
+            where A: HasAfEnum + ImplicitPromote<B>,
+                  B: HasAfEnum + ImplicitPromote<A>,
+                  <A as ImplicitPromote<B>>::Output: HasAfEnum
+        {
+            type Output = Array< <A as ImplicitPromote<B>>::Output >;
 
-            fn $fn_name(self, rhs: Array) -> Array {
+            fn $fn_name(self, rhs: Array<B>) -> Self::Output {
                 $delegate(self, &rhs, false)
             }
         }
 
-        impl<'a, 'b> $op_name<&'a Array> for &'b Array {
-            type Output = Array;
+        impl<'a, 'b, A, B> $op_name<&'a Array<B>> for &'b Array<A>
+            where A: HasAfEnum + ImplicitPromote<B>,
+                  B: HasAfEnum + ImplicitPromote<A>,
+                  <A as ImplicitPromote<B>>::Output: HasAfEnum
+        {
+            type Output = Array< <A as ImplicitPromote<B>>::Output >;
 
-            fn $fn_name(self, rhs: &'a Array) -> Array {
+            fn $fn_name(self, rhs: &'a Array<B>) -> Self::Output {
                 $delegate(self, rhs, false)
             }
         }
@@ -475,20 +573,23 @@ use std::mem;
 use std::ops::{AddAssign, SubAssign, DivAssign, MulAssign, RemAssign};
 use std::ops::{BitAndAssign, BitOrAssign, BitXorAssign, ShlAssign, ShrAssign};
 
-
 macro_rules! arith_assign_func {
     ($op_name:ident, $fn_name:ident, $func: ident) => (
-        impl $op_name<Array> for Array {
-
+        impl<A, B> $op_name<Array<B>> for Array<A>
+            where A: HasAfEnum + ImplicitPromote<B>,
+                  B: HasAfEnum + ImplicitPromote<A>,
+                  <A as ImplicitPromote<B>>::Output: HasAfEnum,
+                  <B as ImplicitPromote<A>>::Output: HasAfEnum
+        {
             #[allow(unused_variables)]
-            fn $fn_name(&mut self, rhs: Array) {
+            fn $fn_name(&mut self, rhs: Array<B>) {
                 let tmp_seq   = Seq::<f32>::default();
                 let mut idxrs = Indexer::new();
                 for n in 0..self.numdims() {
                     idxrs.set_index(&tmp_seq, n, Some(false));
                 }
-                let tmp = assign_gen(self as &Array, &idxrs,
-                                     & $func(self as &Array, &rhs, false));
+                let opres = $func(self as &Array<A>, &rhs, false).cast::<A>();
+                let tmp = assign_gen(self as &Array<A>, &idxrs, &opres);
                 let old = mem::replace(self, tmp);
             }
         }
@@ -505,16 +606,21 @@ arith_assign_func!(ShrAssign, shr_assign, shiftr);
 
 macro_rules! bit_assign_func {
     ($op_name:ident, $fn_name:ident, $func: ident) => (
-        impl $op_name<Array> for Array {
-
+        impl<A,B> $op_name<Array<B>> for Array<A>
+                where A: HasAfEnum + ImplicitPromote<B>,
+                      B: HasAfEnum + ImplicitPromote<A>,
+                      <A as ImplicitPromote<B>>::Output: HasAfEnum,
+                      <B as ImplicitPromote<A>>::Output: HasAfEnum
+        {
             #[allow(unused_variables)]
-            fn $fn_name(&mut self, rhs: Array) {
+            fn $fn_name(&mut self, rhs: Array<B>) {
                 let tmp_seq   = Seq::<f32>::default();
                 let mut idxrs = Indexer::new();
                 for n in 0..self.numdims() {
                     idxrs.set_index(&tmp_seq, n, Some(false));
                 }
-                let tmp = assign_gen(self as &Array, &idxrs, & $func(self as &Array, &rhs, false));
+                let opres = $func(self as &Array<A>, &rhs, false).cast::<A>();
+                let tmp = assign_gen(self as &Array<A>, &idxrs, &opres);
                 let old = mem::replace(self, tmp);
             }
         }
@@ -528,23 +634,17 @@ bit_assign_func!(BitXorAssign, bitxor_assign, bitxor);
 }
 
 ///Implement negation trait for Array
-impl Neg for Array {
-    type Output = Array;
+impl<T> Neg for Array<T>
+    where T: HasAfEnum+Zero+ConstGenerator,
+          <T as ConstGenerator>::OutType: HasAfEnum,
+          <T as ConstGenerator>::OutType: ImplicitPromote<T>,
+          T: ImplicitPromote<<T as ConstGenerator>::OutType>,
+          <<T as ConstGenerator>::OutType as ImplicitPromote<T>>::Output: HasAfEnum
+{
+    type Output = Array< <<T as ConstGenerator>::OutType as ImplicitPromote<T>>::Output >;
 
     fn neg(self) -> Self::Output {
-        match self.get_type() {
-            DType::S64 => (constant_t(Scalar::S64(0 as i64), self.dims(), DType::S64) - self),
-            DType::U64 => (constant_t(Scalar::U64(0 as u64), self.dims(), DType::U64) - self),
-            DType::C32 => (constant_t(Scalar::C32(Complex32::new(0.0, 0.0)), self.dims(), DType::C32) - self),
-            DType::C64 => (constant_t(Scalar::C64(Complex64::new(0.0, 0.0)), self.dims(), DType::C64) - self),
-            DType::F32 => (constant_t(Scalar::F32(0 as f32), self.dims(), DType::F32) - self),
-            DType::F64 => (constant_t(Scalar::F64(0 as f64), self.dims(), DType::F64) - self),
-            DType::B8  => (constant_t(Scalar::B8 (false   ), self.dims(), DType::B8 ) - self),
-            DType::S32 => (constant_t(Scalar::S32(0 as i32), self.dims(), DType::S32) - self),
-            DType::U32 => (constant_t(Scalar::U32(0 as u32), self.dims(), DType::U32) - self),
-            DType::U8  => (constant_t(Scalar::U8 (0 as u8 ), self.dims(), DType::U8 ) - self),
-            DType::S16 => (constant_t(Scalar::S16(0 as i16), self.dims(), DType::S16) - self),
-            DType::U16 => (constant_t(Scalar::U16(0 as u16), self.dims(), DType::U16) - self),
-        }
+        let cnst = constant(T::zero(), self.dims());
+        sub(&cnst, &self, true)
     }
 }
